/*
 * Copyright © 2024 Nick Bowler
 *
 * Helpers for hosts using native Windows threading API.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "windows-once.h"

static DWORD tls_key;
#define tls_key_valid (tls_key != TLS_OUT_OF_INDEXES)

static void init_once_cb(void);

static void *tls_get(void)
{
	return TlsGetValue(tls_key);
}

static BOOL tls_set(void *p)
{
	return TlsSetValue(tls_key, p);
}

static void init_once_with_tls(void)
{
	tls_key = TlsAlloc();
	init_once_cb();
}

static int init_once(void)
{
	static glwthread_once_t ctrl = GLWTHREAD_ONCE_INIT;

	glwthread_once(&ctrl, init_once_with_tls);
	return 1;
}
